@ECHO OFF
SETLOCAL ENABLEDELAYEDEXPANSION
REM ------- this is the only line that needs to be varied ------
SET name=Example&&SET mac=00:11:22:33:44:55&&SET remote=my.domain.dom&&SET wakeport=54144
REM ------ nothing to change below here -------
REM
REM Sleepwalker v4.1 [08 Nov 2023] - by Dominic
REM ===========
REM
REM This Windows program (batch file) can wake up a sleeping (including hibernating) remote Windows machine by sending a magic packet and then start a Remote Desktop connection to that machine.
REM
REM All parameters are set by editing one line, normally the fourth line of the program. It uses wolcmd.exe and portqry.exe, both of which are provided in the standard zip package and should be unzipped/saved to the same location as this batch file. (If SET wakeport=0 is used - see optional variables below - then wolcmd.exe is not required.) [ wolcmd.exe is by Depicus and can be obtained directly from https://www.depicus.com/wake-on-lan/wake-on-lan-cmd.aspx. portqry.exe is by Microsoft and you can read about it and download it at https://support.microsoft.com/kb/310099. ]
REM
REM The remote Windows machine to which you are connecting must have WakeOnLan (aka WOL) enabled both in its BIOS and in Windows Device Manager (on the adapter's 'Power Management' tab i.e. 'Allow this device to wake the computer' and, usually, 'Only allow a magic packet to wake the computer'). Sadly this is often impossible with Wifi network adapters (though there are exceptions), and (at the time of writing) with 10GbE Ethernet adapters.
REM
REM To connect to machines behind a router on a different local network than the machine on which you are running SleepWalker (e.g. over the internet), or (perhaps) if the remote machine is on the same local network but is via one or more intermediate switches, you probably need to have opened or redirected ports on the router to point to the machines, and you probably also need, on the router, to have bound the machines' macs to an IP; this is so the router will allow a magic packet to pass from the internet to a sleeping machine. Some routers simply do not have this capability, in which case this program cannot help you, sorry. Your router must also respond to a ping from the internet.
REM
REM The wakeport setting is not significant to the destination computer but it is usually needed by the router at the destination location to redirect the packet to the correct machine. In effect it identifies which machine the packet should be sent to; this is because most routers block broadcast packets sourced from the internet.
REM
REM It is possible to change the default 3389 Remote Desktop port by a registry fix, but it is better, if your router supports it, to use the router to redirect from another high-numbered port to the local 3389 port; this is more flexible and means the open Remote Desktop connection is less obvious to port scanners.
REM
REM Required Variable:
REM   remote - ip/dns name of remote machine to wake/run Remote Desktop
REM
REM Optional Variables:
REM   mac - mac id of remote machine, if missing this program is just a way to start Remote Desktop
REM   name - appears on the header line of output (unless run with -q)
REM   wakeport - default 9 (set to 0 to skip attempting to wake remote machine)
REM   rdpport - default 3389 (set to 0 to skip launching Remote Desktop)
REM   wakeip - default same as remote
REM   mask - default 255.255.255.255
REM
REM Command line options:
REM   -q: no text unless cannot find wolcmd.exe or portqry.exe or there is some unexpected problem
REM
REM Exit codes:
REM   0 on success, 1 on failure
REM
REM License:
REM   Copyright © 2023 Dominic Raferd. Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with the License. You may obtain a copy of the License at https://www.apache.org/licenses/LICENSE-2.0. Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the License for the specific language governing permissions and limitations under the License.
REM
REM Changelog:
REM   4.1 [08 Nov 2023] - updated help information
REM   4.0 [24 Oct 2022] - remove dual-lan option (which removes ssed.exe dependency), remove ssed.exe from package
REM   3.1 [17 Nov 2016] - switch default wakeonlan port to 9 from 7 (so it works with Wireshark wol.mac/sync/passwd detection), also bugfix - previously in non-verbose mode it used port 7 regardless of setting

SET VERSION=4.0
SET portqryerr=1
IF "%rdpport%" EQU "" SET rdpport=3389
IF "%wakeport%" EQU "" SET wakeport=9
IF "%wakeip%" EQU "" SET wakeip=%remote%
IF "%mac%" NEQ "" SET mac=%mac:-=:%
IF "%mask%" EQU "" SET mask=255.255.255.255
IF /I "%1" NEQ "-q" (
	COLOR 71
	ECHO.
	ECHO SleepWalker v%VERSION% - %name%
	ECHO ================
)
IF "%wakeport%" NEQ "0" (
	REM locate wolcmd.exe program
	wolcmd.exe /? >nul 2>&1
	IF "!ERRORLEVEL!" NEQ "0" (
		"%~dp0wolcmd.exe" /? >nul 2>&1
		IF "!ERRORLEVEL!" NEQ "0" (
			ECHO Sorry, this routine cannot run until you download WolCmd.exe from
			ECHO http://www.depicus.com/wake-on-lan/wake-on-lan-cmd.aspx
			ECHO and save it in the same location as this file!&&GOTO endd
		)
		SET wolcmdpath=%~dp0
	)
)
REM locate portqry.exe program
portqry.exe -n google.com -q >nul 2>&1
IF !ERRORLEVEL! GTR 2 (
	"%~dp0portqry.exe" -n google.com -q >nul 2>&1
	IF !ERRORLEVEL! GTR 2 (
		ping -n 1 google.com >nul
		IF ERRORLEVEL 1 (
			ECHO Unable to reach internet, perhaps your network connection has failed?
			ECHO Aborting...
			EXIT /B 1
		) ELSE (
			ECHO Unable to locate portqry.exe, please obtain this free Microsoft utility ^(try
			ECHO Google^). Lacking portqry.exe this routine runs without testing for connection.
		)
		SET failedportqry=y
	)
	SET portqrypath=%~dp0
)
REM it wastes time to test with portqry first, so send magic packet even if it will not be needed
IF "%mac%" NEQ "" (
	IF "%wakeport%" NEQ "0" (
		IF /I "%1" NEQ "-q" SET /P "dummyvar=!TIME! Sending magic packet to %wakeip% " <nul
		IF /I "%1" EQU "-v" (
			"%wolcmdpath%wolcmd" %mac% %wakeip% %mask% %wakeport%
			IF ERRORLEVEL 1 ECHO There was a problem sending the magic packet to %wakeip%&&GOTO :endd
		) ELSE (
			"%wolcmdpath%wolcmd" %mac% %wakeip% %mask% %wakeport% >NUL
			IF ERRORLEVEL 1 ECHO There was a problem sending the magic packet&&GOTO :endd
		)
		IF /I "%1" NEQ "-q" ECHO.
	)
)
IF "%failedportqry%" EQU "y" GOTO :endd
REM test until we get a connection for quite a long time
IF x%rdpport%==x0 (SET rdpport2=3389) ELSE SET rdpport2=%rdpport%
FOR %%A IN (1 2 3 4 5 6 7 8 9 10 11 12 13 14 15 16 17 18 19 20 21 22 23 24 25 26 27 28 29 30 31 32 33 34 35 36 37 38 39 40) DO (
	IF "!portqryerr!" NEQ "0" (
		IF /I "%1" NEQ "-q" SET /P "dummyvar=!TIME! Testing %remote%: " <nul 
		REM portqry.exe seems to time out after about 21 seconds
		"%portqrypath%portqry.exe" -n %remote% -i -e %rdpport2% -q&&SET portqryerr=!ERRORLEVEL!
		IF "!portqryerr!" NEQ "0" (
			IF /I "%1" NEQ "-q" ECHO No connection yet...
		)
	)
)
IF "!portqryerr!" EQU "0" (
	IF /I "%1" NEQ "-q" ECHO successful connection
) ELSE (
	IF /I "%1" NEQ "-q" ECHO failure - could not confirm remote computer status
)
:endd
REM IF /I "%1" NEQ "-q" PAUSE
IF "!portqryerr!" NEQ "0" EXIT /B 1
SET connected=%remote%
IF NOT x%rdpport%==x0 (
	IF x%rdpport%==x3389 (
		SET machineport=%connected%
	) ELSE (
		SET machineport=%connected%:%rdpport%
	)
	IF /I "%1" NEQ "-q" (
		ECHO !TIME! Starting Remote Desktop connection to !machineport!
		ECHO !TIME! Thank you for using SleepWalker
	)
	START mstsc /v:!machineport!
) ELSE (
	ECHO !TIME! Thank you for using SleepWalker
)
GOTO :eof